@@ -2,14 +2,17 @@ use full_moon::ast::{
     punctuated::{Pair, Punctuated},
     span::ContainedSpan,
     Call, Expression, FunctionArgs, FunctionBody, FunctionCall, FunctionDeclaration, FunctionName,
-    LocalFunction, MethodCall, Parameter, Value,
+    LocalFunction, MethodCall, Parameter, Suffix, Value,
 };
+use full_moon::node::Node;
 use full_moon::tokenizer::{Symbol, Token, TokenKind, TokenReference, TokenType};
 use std::boxed::Box;
 
 use crate::formatters::{
-    trivia_formatter::{self, FormatTriviaType},
-    trivia_util, CodeFormatter,
+    trivia_formatter::{
+        strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia, UpdateTrivia,
+    },
+    trivia_util, CodeFormatter, EndTokenType,
 };
 
 impl CodeFormatter {
@@ -32,20 +35,21 @@ impl CodeFormatter {
             let (parameters_parentheses, return_type) = match function_body.return_type() {
                 Some(return_type) => (
                     function_body.parameters_parentheses().to_owned(),
-                    Some(trivia_formatter::type_specifier_add_trailing_trivia(
-                        return_type.to_owned(),
-                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                    )),
+                    Some(
+                        return_type.update_trailing_trivia(FormatTriviaType::Append(vec![
+                            self.create_newline_trivia()
+                        ])),
+                    ),
                 ),
-                None => {
+                None => (
                     // No return type, so add trivia to the parentheses instead
-                    let parameters_parentheses = trivia_formatter::contained_span_add_trivia(
-                        function_body.parameters_parentheses().to_owned(),
-                        FormatTriviaType::NoChange,
-                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                    );
-                    (parameters_parentheses, None)
-                }
+                    function_body
+                        .parameters_parentheses()
+                        .update_trailing_trivia(FormatTriviaType::Append(vec![
+                            self.create_newline_trivia()
+                        ])),
+                    None,
+                ),
             };
 
             function_body = function_body
@@ -55,19 +59,19 @@ impl CodeFormatter {
 
         #[cfg(not(feature = "luau"))]
         {
-            let parameters_parentheses = trivia_formatter::contained_span_add_trivia(
-                function_body.parameters_parentheses().to_owned(),
-                FormatTriviaType::NoChange,
-                FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-            );
+            let parameters_parentheses = function_body
+                .parameters_parentheses()
+                .update_trailing_trivia(FormatTriviaType::Append(vec![
+                    self.create_newline_trivia()
+                ]));
             function_body = function_body.with_parameters_parentheses(parameters_parentheses);
         };
 
-        let end_token = trivia_formatter::token_reference_add_trivia(
-            function_body.end_token().to_owned(),
-            FormatTriviaType::Append(vec![self.create_indent_trivia(additional_indent_level)]),
-            FormatTriviaType::NoChange,
-        );
+        let end_token = function_body
+            .end_token()
+            .update_leading_trivia(FormatTriviaType::Append(vec![
+                self.create_indent_trivia(additional_indent_level)
+            ]));
 
         (function_token, function_body.with_end_token(end_token))
     }
@@ -283,12 +287,10 @@ impl CodeFormatter {
                         vec![self.create_indent_trivia(end_parens_additional_indent_level)];
 
                     // Add new_line trivia to start_parens
-                    let start_parens_token = crate::fmt_symbol!(self, start_parens, "(");
-                    let start_parens_token = trivia_formatter::token_reference_add_trivia(
-                        start_parens_token,
-                        FormatTriviaType::NoChange,
-                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                    );
+                    let start_parens_token = crate::fmt_symbol!(self, start_parens, "(")
+                        .update_trailing_trivia(FormatTriviaType::Append(vec![
+                            self.create_newline_trivia()
+                        ]));
 
                     let end_parens_token = TokenReference::new(
                         end_parens_leading_trivia,
@@ -348,22 +350,19 @@ impl CodeFormatter {
                         }
 
                         // Add the leading indent for the argument
-                        formatted_argument = trivia_formatter::expression_add_leading_trivia(
-                            formatted_argument,
-                            FormatTriviaType::Append(vec![
-                                self.create_indent_trivia(additional_indent_level)
-                            ]),
-                        );
+                        formatted_argument =
+                            formatted_argument.update_leading_trivia(FormatTriviaType::Append(
+                                vec![self.create_indent_trivia(additional_indent_level)],
+                            ));
 
                         let punctuation = match argument.punctuation() {
                             Some(punctuation) => {
                                 // Continue adding a comma and a new line for multiline function args
                                 let symbol = crate::fmt_symbol!(self, punctuation, ",");
-                                let symbol = trivia_formatter::token_reference_add_trivia(
-                                    symbol,
-                                    FormatTriviaType::NoChange,
-                                    FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                                );
+                                let symbol =
+                                    symbol.update_trailing_trivia(FormatTriviaType::Append(vec![
+                                        self.create_newline_trivia(),
+                                    ]));
 
                                 Some(symbol)
                             }
@@ -408,16 +407,9 @@ impl CodeFormatter {
                     // Recreate parentheses with the comments removed from the opening parens
                     // and all the comments placed at the end of the closing parens
                     let parentheses = ContainedSpan::new(
-                        trivia_formatter::token_reference_add_trivia(
-                            start_parens.to_owned(),
-                            FormatTriviaType::NoChange,
-                            FormatTriviaType::Replace(vec![]),
-                        ),
-                        trivia_formatter::token_reference_add_trivia(
-                            end_parens.to_owned(),
-                            FormatTriviaType::NoChange,
-                            FormatTriviaType::Append(parens_comments),
-                        ),
+                        start_parens.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+                        end_parens
+                            .update_trailing_trivia(FormatTriviaType::Append(parens_comments)),
                     );
 
                     FunctionArgs::Parentheses {
@@ -440,14 +432,11 @@ impl CodeFormatter {
                     trivia_util::get_expression_trailing_comments(&new_expression);
 
                 // Create parentheses, and add the trailing comments to the end of the parentheses
-                let parentheses = trivia_formatter::contained_span_add_trivia(
-                    ContainedSpan::new(
-                        TokenReference::symbol("(").unwrap(),
-                        TokenReference::symbol(")").unwrap(),
-                    ),
-                    FormatTriviaType::NoChange,
-                    FormatTriviaType::Append(comments_buffer),
-                );
+                let parentheses = ContainedSpan::new(
+                    TokenReference::symbol("(").unwrap(),
+                    TokenReference::symbol(")").unwrap(),
+                )
+                .update_trailing_trivia(FormatTriviaType::Append(comments_buffer));
 
                 arguments.push(Pair::new(new_expression, None)); // Only single argument, so no trailing comma
 
@@ -470,14 +459,11 @@ impl CodeFormatter {
                     trivia_util::get_expression_trailing_comments(&new_expression);
 
                 // Create parentheses, and add the trailing comments to the end of the parentheses
-                let parentheses = trivia_formatter::contained_span_add_trivia(
-                    ContainedSpan::new(
-                        TokenReference::symbol("(").unwrap(),
-                        TokenReference::symbol(")").unwrap(),
-                    ),
-                    FormatTriviaType::NoChange,
-                    FormatTriviaType::Append(comments_buffer),
-                );
+                let parentheses = ContainedSpan::new(
+                    TokenReference::symbol("(").unwrap(),
+                    TokenReference::symbol(")").unwrap(),
+                )
+                .update_trailing_trivia(FormatTriviaType::Append(comments_buffer));
 
                 arguments.push(Pair::new(new_expression, None)); // Only single argument, so no trailing comma
 
@@ -502,9 +488,65 @@ impl CodeFormatter {
         let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
         let trailing_trivia = vec![self.create_newline_trivia()];
 
-        let (formatted_parameters, multiline_params) = self.format_parameters(function_body);
+        // Check if the parameters should be placed across multiple lines
+        let multiline_params = {
+            #[cfg(feature = "luau")]
+            let mut type_specifiers = function_body.type_specifiers();
 
-        let mut parameters_parentheses = match multiline_params {
+            // Check whether they contain comments
+            let contains_comments = function_body.parameters().pairs().any(|pair| {
+                let contains_comments = pair
+                    .punctuation()
+                    .map_or(false, |punc| trivia_util::token_contains_comments(punc))
+                    || trivia_util::contains_comments(pair.value());
+                #[cfg(feature = "luau")]
+                let type_specifier_comments = type_specifiers
+                    .next()
+                    .flatten()
+                    .map_or(false, |type_specifier| {
+                        trivia_util::contains_comments(type_specifier)
+                    });
+                #[cfg(not(feature = "luau"))]
+                let type_specifier_comments = false;
+                contains_comments || type_specifier_comments
+            });
+
+            contains_comments
+                || {
+                    // Check the length of the parameters. We need to format them first onto a single line to check if required
+                    let types_length: usize;
+
+                    #[cfg(feature = "luau")]
+                    {
+                        types_length = function_body
+                            .type_specifiers()
+                            .chain(std::iter::once(function_body.return_type())) // Include optional return type
+                            .map(|x| {
+                                x.map_or(0, |specifier| {
+                                    self.format_type_specifier(specifier).to_string().len()
+                                })
+                            })
+                            .sum::<usize>()
+                    }
+                    #[cfg(not(feature = "luau"))]
+                    {
+                        types_length = 0
+                    }
+
+                    let line_length = self
+                    .format_singleline_parameters(function_body)
+                    .to_string()
+                    .len()
+                        + 2 // Account for the parentheses around the parameters
+                        + types_length // Account for type specifiers and return type
+                        + self.indent_level * self.config.indent_width
+                        + self.get_range_indent_increase(CodeFormatter::get_token_range(function_body.parameters_parentheses().tokens().0)).unwrap_or(0) * self.config.indent_width;
+
+                    line_length > self.config.column_width
+                }
+        };
+
+        let (formatted_parameters, mut parameters_parentheses) = match multiline_params {
             true => {
                 // TODO: This is similar to multiline in FunctionArgs, can we resolve?
                 // Format start and end brace properly with correct trivia
@@ -521,12 +563,10 @@ impl CodeFormatter {
                 ];
 
                 // Add new_line trivia to start_parens
-                let start_parens_token = crate::fmt_symbol!(self, start_parens, "(");
-                let start_parens_token = trivia_formatter::token_reference_add_trivia(
-                    start_parens_token,
-                    FormatTriviaType::NoChange,
-                    FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                );
+                let start_parens_token = crate::fmt_symbol!(self, start_parens, "(")
+                    .update_trailing_trivia(FormatTriviaType::Append(vec![
+                        self.create_newline_trivia()
+                    ]));
 
                 let end_parens_token = TokenReference::new(
                     end_parens_leading_trivia,
@@ -536,16 +576,22 @@ impl CodeFormatter {
                     vec![],
                 );
 
-                ContainedSpan::new(
-                    start_parens_token,
-                    self.format_symbol(end_parens, &end_parens_token),
+                (
+                    self.format_multiline_parameters(function_body),
+                    ContainedSpan::new(
+                        start_parens_token,
+                        self.format_symbol(end_parens, &end_parens_token),
+                    ),
                 )
             }
-            false => self.format_contained_span(function_body.parameters_parentheses()),
+            false => (
+                self.format_singleline_parameters(function_body),
+                self.format_contained_span(function_body.parameters_parentheses()),
+            ),
         };
 
         #[cfg(feature = "luau")]
-        let mut type_specifiers;
+        let type_specifiers;
         #[cfg(feature = "luau")]
         let return_type;
         #[allow(unused_mut)]
@@ -553,24 +599,19 @@ impl CodeFormatter {
 
         #[cfg(feature = "luau")]
         {
-            type_specifiers = Vec::new();
-            for specifier in function_body.type_specifiers() {
-                let formatted_specifier = match specifier {
-                    Some(specifier) => Some(self.format_type_specifier(specifier)),
-                    None => None,
-                };
-                type_specifiers.push(formatted_specifier);
-            }
+            type_specifiers = function_body
+                .type_specifiers()
+                .map(|x| x.map(|specifier| self.format_type_specifier(specifier)))
+                .collect();
 
             return_type = match function_body.return_type() {
                 Some(return_type) => Some({
                     let formatted = self.format_type_specifier(return_type);
                     if add_trivia {
                         added_trailing_trivia = true;
-                        trivia_formatter::type_specifier_add_trailing_trivia(
-                            formatted,
-                            FormatTriviaType::Append(trailing_trivia.to_owned()),
-                        )
+                        formatted.update_trailing_trivia(FormatTriviaType::Append(
+                            trailing_trivia.to_owned(),
+                        ))
                     } else {
                         formatted
                     }
@@ -580,21 +621,18 @@ impl CodeFormatter {
         }
 
         if !added_trailing_trivia && add_trivia {
-            parameters_parentheses = trivia_formatter::contained_span_add_trivia(
-                parameters_parentheses,
-                FormatTriviaType::NoChange,
-                FormatTriviaType::Append(trailing_trivia.to_owned()),
-            )
+            parameters_parentheses = parameters_parentheses
+                .update_trailing_trivia(FormatTriviaType::Append(trailing_trivia.to_owned()))
         }
 
         let end_token = if add_trivia {
-            trivia_formatter::token_reference_add_trivia(
-                self.format_end_token(function_body.end_token()),
-                FormatTriviaType::Append(leading_trivia),
-                FormatTriviaType::Append(trailing_trivia),
-            )
+            self.format_end_token(function_body.end_token(), EndTokenType::BlockEnd)
+                .update_trivia(
+                    FormatTriviaType::Append(leading_trivia),
+                    FormatTriviaType::Append(trailing_trivia),
+                )
         } else {
-            self.format_end_token(function_body.end_token())
+            self.format_end_token(function_body.end_token(), EndTokenType::BlockEnd)
         };
 
         let function_body = function_body
@@ -617,10 +655,94 @@ impl CodeFormatter {
         function_call: &FunctionCall<'ast>,
     ) -> FunctionCall<'ast> {
         let formatted_prefix = self.format_prefix(function_call.prefix());
-        let formatted_suffixes = function_call
-            .suffixes()
-            .map(|x| self.format_suffix(x))
-            .collect();
+
+        let num_suffixes = function_call.suffixes().count();
+        let should_hang = {
+            // Hang if there is atleast more than one method call suffix
+            if function_call
+                .suffixes()
+                .filter(|x| matches!(x, Suffix::Call(Call::MethodCall(_))))
+                .count()
+                > 1
+            {
+                // Check if either a), we are surpassing the column width
+                // Or b), one of the INTERNAL (not the last call) method call's arguments is multiline [function/table]
+
+                // Create a temporary formatted version of suffixes to use for this check
+                let formatted_suffixes = function_call
+                    .suffixes()
+                    .map(|x| self.format_suffix(x))
+                    .collect();
+                let preliminary_function_call = FunctionCall::new(formatted_prefix.to_owned())
+                    .with_suffixes(formatted_suffixes);
+
+                let outcome = if strip_trivia(&preliminary_function_call)
+                    .to_string()
+                    .lines()
+                    .next()
+                    .expect("no lines")
+                    .len()
+                    > self.config.column_width
+                {
+                    true
+                } else {
+                    let suffixes = preliminary_function_call.suffixes().enumerate();
+                    let mut contains_newline = false;
+                    for (idx, suffix) in suffixes {
+                        // Check to see whether this suffix is an "internal" method call suffix
+                        // i.e. we are not at the last MethodCall suffix
+                        let mut remaining_suffixes =
+                            preliminary_function_call.suffixes().skip(idx + 1);
+                        if remaining_suffixes
+                            .any(|x| matches!(x, Suffix::Call(Call::MethodCall(_))))
+                        {
+                            if matches!(suffix, Suffix::Call(Call::MethodCall(_)))
+                                && strip_trivia(suffix).to_string().contains('\n')
+                            {
+                                contains_newline = true;
+                                break;
+                            }
+                        }
+                    }
+
+                    contains_newline
+                };
+
+                outcome
+            } else {
+                false
+            }
+        };
+
+        let mut formatted_suffixes = Vec::with_capacity(num_suffixes);
+        for suffix in function_call.suffixes() {
+            // Calculate the range before formatting, otherwise it will reset to (0,0)
+            let range = (
+                suffix.start_position().expect("no suffix position").bytes(),
+                suffix.end_position().expect("no suffix position").bytes(),
+            );
+
+            let indent_level = if should_hang && matches!(suffix, Suffix::Call(Call::MethodCall(_)))
+            {
+                // Calculate the indentation level for the suffix, before we add the range into the indent ranges
+                let indent_level = self.get_range_indent_increase(range).unwrap_or(0);
+                self.add_indent_range(range);
+                Some(indent_level + 1)
+            } else {
+                None
+            };
+
+            let mut suffix = self.format_suffix(suffix);
+
+            if indent_level.is_some() {
+                suffix = suffix.update_leading_trivia(FormatTriviaType::Append(vec![
+                    self.create_newline_trivia(),
+                    self.create_indent_trivia(indent_level),
+                ]));
+            }
+
+            formatted_suffixes.push(suffix);
+        }
 
         FunctionCall::new(formatted_prefix).with_suffixes(formatted_suffixes)
     }
@@ -672,11 +794,9 @@ impl CodeFormatter {
         );
         let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
 
-        let function_token = trivia_formatter::token_reference_add_trivia(
-            crate::fmt_symbol!(self, function_declaration.function_token(), "function "),
-            FormatTriviaType::Append(leading_trivia),
-            FormatTriviaType::NoChange,
-        );
+        let function_token =
+            crate::fmt_symbol!(self, function_declaration.function_token(), "function ")
+                .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
         let formatted_function_name = self.format_function_name(function_declaration.name());
         let formatted_function_body = self.format_function_body(function_declaration.body(), true);
 
@@ -696,11 +816,8 @@ impl CodeFormatter {
         );
         let leading_trivia = vec![self.create_indent_trivia(additional_indent_level)];
 
-        let local_token = trivia_formatter::token_reference_add_trivia(
-            crate::fmt_symbol!(self, local_function.local_token(), "local "),
-            FormatTriviaType::Append(leading_trivia),
-            FormatTriviaType::NoChange,
-        );
+        let local_token = crate::fmt_symbol!(self, local_function.local_token(), "local ")
+            .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
 
         let function_token = crate::fmt_symbol!(self, local_function.function_token(), "function ");
         let formatted_name = self.format_token_reference(local_function.name());
@@ -733,59 +850,60 @@ impl CodeFormatter {
         }
     }
 
-    // Checks whether the input Parameter contains comments
-    fn parameter_contains_comments(parameter: &Parameter<'_>) -> bool {
-        match parameter {
-            Parameter::Ellipse(token) | Parameter::Name(token) => {
-                trivia_util::token_contains_comments(token)
-            }
-            other => panic!("unknown node {:?}", other),
-        }
-    }
-    /// Utilises the FunctionBody iterator to format a list of Parameter nodes
-    /// Returns the formatted Punctuated sequence of parameters, and a bool indicating whether the parameters were forced onto multiple lines
-    fn format_parameters<'ast>(
+    /// Formats the [`Parameters`] in the provided [`FunctionBody`] onto a single line.
+    fn format_singleline_parameters<'ast>(
         &mut self,
         function_body: &FunctionBody<'ast>,
-    ) -> (Punctuated<'ast, Parameter<'ast>>, bool) {
+    ) -> Punctuated<'ast, Parameter<'ast>> {
         let mut formatted_parameters = Punctuated::new();
-        let force_multiline = function_body.parameters().pairs().any(|pair| {
-            pair.punctuation()
-                .map_or(false, |punc| trivia_util::token_contains_comments(punc))
-                || CodeFormatter::parameter_contains_comments(pair.value())
-        });
 
-        let parameters_iterator = function_body.parameters().pairs();
-        for pair in parameters_iterator {
-            let formatted_parameter = {
-                let param = self.format_parameter(pair.value());
-                if force_multiline {
-                    trivia_formatter::parameter_add_trivia(
-                        param,
-                        FormatTriviaType::Append(vec![self.create_indent_trivia(Some(1))]),
-                        FormatTriviaType::NoChange,
-                    )
-                } else {
-                    param
-                }
-            };
-
-            let formatted_punctuation = match pair.punctuation() {
-                Some(punctuation) => Some(match force_multiline {
-                    true => trivia_formatter::token_reference_add_trivia(
-                        // Create a comma with no trailing space, and instead we will add a newline character
-                        crate::fmt_symbol!(self, punctuation, ","),
-                        FormatTriviaType::NoChange,
-                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
-                    ),
-                    // Create a comma, with a trailing space at the end
-                    false => crate::fmt_symbol!(self, punctuation, ", "),
-                }),
+        for pair in function_body.parameters().pairs() {
+            let parameter = self.format_parameter(pair.value());
+            let punctuation = match pair.punctuation() {
+                Some(punctuation) => Some(crate::fmt_symbol!(self, punctuation, ", ")),
                 None => None,
             };
 
-            formatted_parameters.push(Pair::new(formatted_parameter, formatted_punctuation));
+            formatted_parameters.push(Pair::new(parameter, punctuation));
         }
-        (formatted_parameters, force_multiline)
+
+        formatted_parameters
+    }
+
+    /// Formats the [`Parameters`] in the provided [`FunctionBody`], split across multiple lines.
+    fn format_multiline_parameters<'ast>(
+        &mut self,
+        function_body: &FunctionBody<'ast>,
+    ) -> Punctuated<'ast, Parameter<'ast>> {
+        let mut formatted_parameters = Punctuated::new();
+
+        for pair in function_body.parameters().pairs() {
+            // Calculate indent increase
+            let additional_indent_level = self.get_range_indent_increase(match pair.value() {
+                Parameter::Name(token) | Parameter::Ellipse(token) => {
+                    CodeFormatter::get_token_range(token)
+                }
+                other => panic!("unknown node {:?}", other),
+            });
+
+            let parameter = self.format_parameter(pair.value()).update_leading_trivia(
+                FormatTriviaType::Append(vec![
+                    self.create_indent_trivia(Some(additional_indent_level.unwrap_or(0) + 1))
+                ]),
+            );
+
+            let punctuation = match pair.punctuation() {
+                Some(punctuation) => Some(
+                    crate::fmt_symbol!(self, punctuation, ",").update_trailing_trivia(
+                        FormatTriviaType::Append(vec![self.create_newline_trivia()]),
+                    ),
+                ),
+                None => None,
+            };
+
+            formatted_parameters.push(Pair::new(parameter, punctuation))
+        }
+
+        formatted_parameters
     }
 }
